/**
 *
 * Magnolia SimpleMedia Module (http://www.openmindlab.com/lab/products/media.html)
 * Copyright (C)2008 - 2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.sourceforge.openutils.mgnlmedia.media.tags.el;

import info.magnolia.cms.beans.runtime.FileProperties;
import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.NodeData;
import info.magnolia.cms.util.NodeDataUtil;
import info.magnolia.context.MgnlContext;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.jcr.ItemNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.query.InvalidQueryException;

import net.sourceforge.openutils.mgnlmedia.media.configuration.MediaConfigurationManager;
import net.sourceforge.openutils.mgnlmedia.media.configuration.MediaTypeConfiguration;
import net.sourceforge.openutils.mgnlmedia.media.lifecycle.MediaModule;
import net.sourceforge.openutils.mgnlmedia.media.types.impl.BaseTypeHandler;
import net.sourceforge.openutils.mgnlmedia.media.utils.ImageUtils;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Class that holds media el functions methods
 * @author molaschi
 * @version $Id: MediaEl.java 1628 2010-01-09 17:38:55Z fgiust $
 */
public final class MediaEl
{

    private static MediaConfigurationManager mcm = MediaConfigurationManager.getInstance();

    private static final Logger log = LoggerFactory.getLogger(MediaEl.class);

    private static final String[] EMPTY_STRING_ARRAY = new String[]{};

    private MediaEl()
    {

    }

    /**
     * Get content node for media
     * @param obj content node or node UUID
     * @return content node
     */
    public static Content getNode(Object obj)
    {
        if (obj == null)
        {
            return null;
        }
        if (obj instanceof String)
        {
            String uuid = (String) obj;

            if (StringUtils.isBlank(uuid))
            {
                return null;
            }

            HierarchyManager hm = MgnlContext.getHierarchyManager(MediaModule.REPO);
            try
            {
                return hm.getContentByUUID(StringUtils.trim(uuid));
            }
            catch (ItemNotFoundException e)
            {
                log.debug("Node with UUID \"" + uuid + "\" not found");
            }
            catch (RepositoryException e)
            {
                log.error(e.getClass().getName() + " getting node with UUID \"" + uuid + "\"", e);
            }
        }
        else if (obj instanceof Content)
        {
            return (Content) obj;
        }
        return null;
    }

    /**
     * Get localized description for a media. If the locale is "it", it searches for a nodedata called description-it,
     * or (if not found) description-en or (if not found) description.
     * @param media media
     * @param locale language
     * @return description
     */
    public static String getDescription(Content media, Locale locale)
    {
        if (media == null)
        {
            return null;
        }
        if (locale != null)
        {
            MgnlContext.setLocale(locale);
        }
        MediaTypeConfiguration mtc = mcm.getMediaTypeConfigurationFromMedia(media);
        if (mtc == null)
        {
            return null;
        }
        return mtc.getHandler().getDescription(media);
    }

    /**
     * Get description for a media
     * @param media media
     * @return description
     */
    public static String getDescription(Content media)
    {
        return getDescription(media, null);
    }

    /**
     * Get localized title for a media. If the locale is "it", it searches for a nodedata called title-it, or (if not
     * found) title-en or (if not found) title.
     * @param media media
     * @param locale language
     * @return title
     */
    public static String getTitle(Content media, Locale locale)
    {
        if (media == null)
        {
            return null;
        }
        if (locale != null)
        {
            MgnlContext.setLocale(locale);
        }
        MediaTypeConfiguration mtc = mcm.getMediaTypeConfigurationFromMedia(media);
        if (mtc != null)
        {
            return mtc.getHandler().getTitle(media);
        }
        return null;
    }

    /**
     * Get title for a media
     * @param media media
     * @return title
     */
    public static String getTitle(Content media)
    {
        return getTitle(media, null);
    }

    /**
     * Get localized tags for a media. If the locale is "it", it searches for a nodedata called tags-it, or (if not
     * found) tags-en or (if not found) tags.
     * @param media media
     * @param locale language
     * @return tags
     */
    public static String[] getTags(Content media, Locale locale)
    {
        if (media == null)
        {
            return null;
        }
        if (locale != null)
        {
            MgnlContext.setLocale(locale);
        }
        MediaTypeConfiguration mtc = mcm.getMediaTypeConfigurationFromMedia(media);
        String tags = mtc.getHandler().getTags(media);
        if (tags != null)
        {
            return StringUtils.split(tags, ",");
        }
        return null;
    }

    /**
     * Get tags for a media
     * @param media media
     * @return tags
     */
    public static String[] getTags(Content media)
    {
        return getTags(media, null);
    }

    /**
     * Get url for a media
     * @param media media
     * @return url
     */
    public static String getUrl(Content media)
    {
        return getUrl2(media, Collections.<String, String> emptyMap());
    }

    /**
     * Get url for a media, passing some parameters
     * @param media media
     * @param options optional parameters
     * @return url
     */
    public static String getUrl2(Content media, Map<String, String> options)
    {
        if (media == null)
        {
            return null;
        }
        MediaTypeConfiguration mtc = mcm.getMediaTypeConfigurationFromMedia(media);
        return mtc.getHandler().getUrl(media, options);
    }

    /**
     * Get url to thumbnail
     * @param media media to get the url for
     * @return the thumbnail url for this media, null otherwise
     */
    public static String getThumbnail(Content media)
    {
        if (media == null)
        {
            return null;
        }
        MediaTypeConfiguration mtc = mcm.getMediaTypeConfigurationFromMedia(media);
        return mtc.getHandler().getThumbnailUrl(media);
    }

    /**
     * Get media type
     * @param media media to get the type
     * @return the type of this media if existing, null otherwise
     */
    public static String getType(Content media)
    {
        if (media == null)
        {
            return null;
        }
        return NodeDataUtil.getString(media, "type");
    }

    /**
     * Get all resolution strings (i.e. 'o200x350;background=45A97B') that generates cached resolutions
     * @param media media to get the resolutions
     * @return all resolution strings
     */
    @SuppressWarnings("unchecked")
    public static String[] listResolutions(Content media)
    {

        List<String> res = new ArrayList<String>();

        Content resolutions = media.getChildByName("resolutions");

        for (NodeData item : (Collection<NodeData>) resolutions.getNodeDataCollection())
        {
            if (item.getName().startsWith("res-"))
            {
                if (item.getAttribute(ImageUtils.RESOLUTION_PROPERTY) != null)
                {
                    res.add(item.getAttribute(ImageUtils.RESOLUTION_PROPERTY));
                }
                else
                {
                    res.add(StringUtils.substringAfter(item.getName(), "-"));
                }
            }
        }

        return res.toArray(new String[res.size()]);

    }

    /**
     * Get url for a resolution
     * @param media media
     * @param resolution resolution
     * @return url
     */
    public static String getResolutionUrl(Content media, String resolution)
    {
        FileProperties prop = new FileProperties(media, BaseTypeHandler.ORGINAL_NODEDATA_NAME);
        Integer width = NumberUtils.toInt(prop.getProperty(FileProperties.PROPERTY_WIDTH));
        Integer height = NumberUtils.toInt(prop.getProperty(FileProperties.PROPERTY_HEIGHT));
        Point size = ImageUtils.parseForSize(resolution);
        if (width == size.x && height == size.y)
        {
            return mcm.getURIMappingPrefix() + prop.getProperty(FileProperties.PATH);
        }

        if (media == null || !ImageUtils.checkOrCreateResolution(media, resolution, null))
        {
            return null;
        }

        Content resolutions = media.getChildByName("resolutions");

        try
        {
            String resString = "res-" + ImageUtils.getResolutionPath(resolution);
            if (resolutions != null && resolutions.hasNodeData(resString))
            {
                String resPath = new FileProperties(resolutions, resString).getProperty(FileProperties.PATH);

                return mcm.getURIMappingPrefix() + resPath;

            }
        }
        catch (RepositoryException e)
        {
            log.debug(e.getMessage(), e);
        }
        return null;
    }

    /**
     * Get size of an image
     * @param media media
     * @param resolution resolution for witch calculate size
     * @return size of an image
     */
    public static int[] getSize(Content media, String resolution)
    {
        if (media != null)
        {

            NodeData res = null;
            if ("original".equals(resolution))
            {
                res = media.getNodeData("original");
            }
            else
            {
                Content resolutions = media.getChildByName("resolutions");
                try
                {
                    if (resolutions.hasNodeData(ImageUtils.getResolutionPath("res-" + resolution)))
                    {
                        res = resolutions.getNodeData(ImageUtils.getResolutionPath("res-" + resolution));
                    }
                    else
                    {
                        res = null;
                    }
                }
                catch (RepositoryException e)
                {
                    res = null;
                }
            }

            if (res != null)
            {
                return new int[]{
                    NumberUtils.toInt(res.getAttribute(FileProperties.PROPERTY_WIDTH)),
                    NumberUtils.toInt(res.getAttribute(FileProperties.PROPERTY_HEIGHT)) };
            }
        }

        return new int[]{-1, -1 };
    }

    /**
     * Get url to media preview
     * @param media media
     * @return the preview url for this media if existing, null otherwise
     */
    public static String getPreview(Content media)
    {
        if (media == null)
        {
            return null;
        }
        MediaTypeConfiguration mtc = mcm.getMediaTypeConfigurationFromMedia(media);
        return mtc.getHandler().getPreviewUrl(media);
    }

    /**
     * Get an array of String(s) containing a list of web pages where this media is used, an empty array otherwise
     * @param media media to search in web pages
     * @return an array of String(s) containing a list of web pages where this media is used, an empty array otherwise
     * @throws IllegalArgumentException if media is null
     */
    public static String[] findMediaUsedInWebPages(Content media)
    {
        if (media == null)
        {
            String msg = "media cannot be null";
            log.error(msg);

            throw new IllegalArgumentException(msg);
        }
        try
        {
            List<String> retVal = mcm.getUsedInWebPages(media.getUUID());
            return (CollectionUtils.isNotEmpty(retVal) ? retVal.toArray(new String[retVal.size()]) : EMPTY_STRING_ARRAY);
        }
        catch (InvalidQueryException e)
        {
            log.error("JCR Query returned the following error: ", e);
            // ignore this and return an empty List
        }
        catch (RepositoryException e)
        {
            log.error("JCR Repository returned the following error: ", e);
            // ignore this and return an empty List
        }
        return EMPTY_STRING_ARRAY;
    }
}
