/**
 *
 * Magnolia SimpleMedia Module (http://www.openmindlab.com/lab/products/media.html)
 * Copyright (C)2008 - 2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.sourceforge.openutils.mgnlmedia.media.types.impl;

import info.magnolia.cms.beans.runtime.Document;
import info.magnolia.cms.beans.runtime.FileProperties;
import info.magnolia.cms.beans.runtime.MultipartForm;
import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.NodeData;
import info.magnolia.cms.security.AccessDeniedException;
import info.magnolia.cms.util.NodeDataUtil;
import info.magnolia.context.MgnlContext;
import info.magnolia.module.admininterface.SaveHandlerImpl;

import java.io.File;
import java.util.Collection;
import java.util.Locale;
import java.util.Map;

import javax.jcr.RepositoryException;
import javax.servlet.http.HttpServletRequest;

import net.sourceforge.openutils.mgnlmedia.media.configuration.MediaConfigurationManager;
import net.sourceforge.openutils.mgnlmedia.media.lifecycle.MediaModule;
import net.sourceforge.openutils.mgnlmedia.media.types.MediaTypeHandler;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Base implementation of MediaTypeHandler for common used method
 * @author molaschi
 * @version $Id: BaseTypeHandler.java 1628 2010-01-09 17:38:55Z fgiust $
 */
public abstract class BaseTypeHandler implements MediaTypeHandler
{

    /**
     * Nodedata name where original media content is saved
     */
    public static final String ORGINAL_NODEDATA_NAME = "original";

    /**
     * Logger.
     */
    private Logger log = LoggerFactory.getLogger(BaseTypeHandler.class);

    private String type = null;

    /**
     * {@inheritDoc}
     */
    public void init(Content typeDefinitionNode)
    {
        type = typeDefinitionNode.getName();
    }

    /**
     * {@inheritDoc}
     */
    public String getNewNodeName(MultipartForm form, HttpServletRequest request)
    {
        return request.getParameter(ORGINAL_NODEDATA_NAME + "_" + FileProperties.PROPERTY_FILENAME);
    }

    public String getPreviewImageNodeDataName()
    {
        return ORGINAL_NODEDATA_NAME;
    }

    /**
     * {@inheritDoc}
     */
    public boolean onSavingPropertyMedia(Content media, Content parentNode, Content configNode, String name,
        HttpServletRequest request, MultipartForm form, int type, int valueType, int isRichEditValue, int encoding)
        throws RepositoryException, AccessDeniedException
    {
        String uuid = request.getParameter(name);

        NodeData nd = NodeDataUtil.getOrCreate(parentNode, name);
        nd.setValue(uuid);

        return true;
    }

    /**
     * {@inheritDoc}
     */
    public void saveFromZipFile(Content media, File f, String cleanFileName, String extension)
        throws AccessDeniedException, RepositoryException
    {
        Document doc = new Document(f, type + extension);
        doc.setExtention(extension);
        SaveHandlerImpl.saveDocument(media, doc, ORGINAL_NODEDATA_NAME, cleanFileName, cleanFileName
            + "_"
            + FileProperties.PROPERTY_TEMPLATE);
        this.onPostSave(media);
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public boolean onPostSave(Content media)
    {
        try
        {
            if (media.hasContent("resolutions"))
            {
                Collection<NodeData> nodedatas = media.getChildByName("resolutions").getNodeDataCollection();
                for (NodeData nd : nodedatas)
                {
                    nd.delete();
                }
                media.save();
            }

            if (MediaModule.getInstance().isAutoactivate())
            {
                media.getMetaData().setActivated();
                media.save();
            }
        }
        catch (RepositoryException ex)
        {
            log.error("Error removing resolutions", ex);
        }

        Collection<NodeData> nds = media.getNodeDataCollection("tags*");
        for (NodeData nd : nds)
        {
            if (nd.getString() != null)
            {
                try
                {
                    nd.setValue(nd.getString().toLowerCase());
                    nd.save();
                }
                catch (RepositoryException ex)
                {
                    log.warn("Error putting property to lower case", ex);
                }
            }
        }
        return true;
    }

    /**
     * Get the default "original" nodedata
     * @param media media
     * @return default nodedata
     */
    protected NodeData getOriginalFileNodeData(Content media)
    {
        return media.getNodeData(ORGINAL_NODEDATA_NAME);
    }

    /**
     * {@inheritDoc}
     */
    public String getExtension(Content media)
    {
        return getOriginalFileNodeData(media).getAttribute(FileProperties.PROPERTY_EXTENSION);
    }

    /**
     * {@inheritDoc}
     */
    public String getFilename(Content media)
    {
        return getOriginalFileNodeData(media).getAttribute(FileProperties.PROPERTY_FILENAME);
    }

    /**
     * {@inheritDoc}
     */
    public String getFullFilename(Content media)
    {
        return getFilename(media)
            + (StringUtils.isNotBlank(getExtension(media)) ? "." + getExtension(media) : StringUtils.EMPTY);
    }

    /**
     * {@inheritDoc}
     */
    public String getUrl(Content media, Map<String, String> options)
    {
        return MediaConfigurationManager.getInstance().getURIMappingPrefix()
            + media.getHandle()
            + "/"
            + ORGINAL_NODEDATA_NAME
            + "/"
            + getFullFilename(media);
    }

    /**
     * {@inheritDoc}
     */
    public String getUrl(Content media)
    {
        return getUrl(media, null);
    }

    /**
     * {@inheritDoc}
     */
    public String getPreviewUrl(Content media)
    {
        return getUrl(media);
    }

    /**
     * Get the nodedata value for a given name and current locale searching first for [propertyname]-[lang]_[COUNTRY]
     * then for [propertyname]-[lang] then for [propertyname]-en then for [propertyname].<br />
     * i.e for the property title and for a locale it,IT:
     * <ol>
     * <li>search for nodedata "title-it_IT"</li>
     * <li>search for nodedata "title-it"</li>
     * <li>search for nodedata "title-en"</li>
     * <li>search for nodedata "title"</li>
     * </ol>
     * @param media media
     * @param property base property name
     * @return
     */
    protected String getPropertyLocalized(Content media, String property)
    {
        Locale locale = MgnlContext.getLocale();
        String language = locale.getLanguage().toLowerCase();
        String languageCountry = language + "_" + locale.getCountry().toLowerCase();
        try
        {
            if (LANGUAGES.contains(languageCountry) && media.hasNodeData(property + "-" + languageCountry))
            {
                return NodeDataUtil.getString(media, property + "-" + languageCountry);
            }
            else if (LANGUAGES.contains(language) && media.hasNodeData(property + "-" + language))
            {
                return NodeDataUtil.getString(media, property + "-" + language);
            }
            else if (media.hasNodeData(property + "-en"))
            {
                return NodeDataUtil.getString(media, property + "-" + language);
            }
        }
        catch (RepositoryException ex)
        {
            log.error(ex.getMessage(), ex);
        }
        return NodeDataUtil.getString(media, property);
    }

    /**
     * {@inheritDoc}
     */
    public String getTitle(Content media)
    {
        return getPropertyLocalized(media, "title");
    }

    /**
     * {@inheritDoc}
     */
    public String getTags(Content media)
    {
        return getPropertyLocalized(media, "tags");
    }

    /**
     * {@inheritDoc}
     */
    public String getDescription(Content media)
    {
        return getPropertyLocalized(media, "description");
    }

    /**
     * {@inheritDoc}
     */
    public String getAbstract(Content media)
    {
        return getPropertyLocalized(media, "abstract");
    }

}
